import 'package:flutter/cupertino.dart';

import 'package:forme/forme.dart';

import 'package:forme_cupertino_fields/forme_cupertino_fields.dart';

typedef FormeDurationFormatter = String Function(
    Duration duration, CupertinoTimerPickerMode mode);

/// used to pick time only
class FormeCupertinoTimerField
    extends ValueField<Duration?, FormeCupertinoTimerFieldModel> {
  FormeCupertinoTimerField({
    Duration? initialValue,
    required String name,
    bool readOnly = false,
    FormeCupertinoTimerFieldModel? model,
    Key? key,
    FormeDecoratorBuilder<Duration?>? decoratorBuilder,
    int? maxLines = 1,
    FormeValueFieldListener<
            Duration?,
            FormeValueFieldController<Duration?,
                FormeCupertinoTimerFieldModel>>?
        listener,
    int? order,
    bool quietlyValidate = false,
  }) : super(
          quietlyValidate: quietlyValidate,
          listener: listener,
          decoratorBuilder: decoratorBuilder,
          key: key,
          model: (model ?? FormeCupertinoTimerFieldModel()).copyWith(
            FormeCupertinoTimerFieldModel(
              textFieldModel: FormeCupertinoTextFieldModel(
                maxLines: maxLines,
              ),
            ),
          ),
          name: name,
          initialValue: initialValue,
          builder: (state) {
            bool readOnly = state.readOnly;
            FocusNode focusNode = state.focusNode;
            TextEditingController textEditingController =
                (state as _FormeCupertinoTimerFieldState).textEditingController;

            void pickDuration() {
              state.duration = null;
              FormeCupertinos.showFormeCupertinoModalPopup(
                context: state.context,
                data: state.model.modelPopupData,
                builder: (context) {
                  return Container(
                    color: CupertinoColors.systemBackground,
                    child: Wrap(children: [
                      Row(
                        mainAxisAlignment: MainAxisAlignment.end,
                        children: [
                          CupertinoButton(
                            onPressed: () {
                              if (state.duration == null) {
                                if (state.value == null) {
                                  state.didChange(Duration.zero);
                                }
                              } else {
                                state.didChange(state.duration);
                              }
                              state.duration = null;
                              Navigator.of(context).pop();
                            },
                            child: (state.model.confirmWidget ??
                                const Icon(
                                  CupertinoIcons.check_mark,
                                )),
                          ),
                        ],
                      ),
                      CupertinoTimerPicker(
                        minuteInterval: state.model.minuteInterval ?? 1,
                        secondInterval: state.model.secondInterval ?? 1,
                        backgroundColor: state.model.backgroundColor,
                        alignment: state.model.alignment ?? Alignment.center,
                        initialTimerDuration: state.value ?? Duration.zero,
                        mode: state.mode,
                        onTimerDurationChanged: (Duration changedtimer) {
                          state.duration = changedtimer;
                        },
                      ),
                    ]),
                  );
                },
              ).whenComplete(() => state.requestFocus());
            }

            return FormeCupertinos.cupertinoTextField(
                textEditingController: textEditingController,
                focusNode: focusNode,
                model: FormeCupertinoTextFieldModel(
                  inputFormatters: [],
                  onTap: readOnly ? () {} : pickDuration,
                  readOnly: true,
                ).copyWith(state.model.textFieldModel ??
                    FormeCupertinoTextFieldModel()));
          },
        );

  @override
  _FormeCupertinoTimerFieldState createState() =>
      _FormeCupertinoTimerFieldState();

  static FormeDurationFormatter defaultDurationFormatter = (v, mode) {
    switch (mode) {
      case CupertinoTimerPickerMode.hm:
        return '${v.inHours.toString().padLeft(2, '0')}:${v.inMinutes.remainder(60).toString().padLeft(2, '0')}';
      case CupertinoTimerPickerMode.ms:
        return '${v.inMinutes.remainder(60).toString().padLeft(2, '0')}:${v.inSeconds.remainder(60).toString().padLeft(2, '0')}';
      case CupertinoTimerPickerMode.hms:
        return '${v.inHours.toString().padLeft(2, '0')}:${v.inMinutes.remainder(60).toString().padLeft(2, '0')}:${v.inSeconds.remainder(60).toString().padLeft(2, '0')}';
    }
  };
}

class _FormeCupertinoTimerFieldState
    extends ValueFieldState<Duration?, FormeCupertinoTimerFieldModel> {
  FormeDurationFormatter get _formatter =>
      model.formatter ?? FormeCupertinoTimerField.defaultDurationFormatter;

  late final TextEditingController textEditingController;

  Duration? duration;

  CupertinoTimerPickerMode get mode =>
      model.mode ?? CupertinoTimerPickerMode.hm;

  @override
  FormeCupertinoTimerField get widget =>
      super.widget as FormeCupertinoTimerField;

  @override
  void afterInitiation() {
    super.afterInitiation();
    textEditingController = TextEditingController(
        text: initialValue == null ? '' : _formatter(initialValue!, mode));
  }

  @override
  void onValueChanged(Duration? value) {
    textEditingController.text = value == null ? '' : _formatter(value, mode);
  }

  @override
  void dispose() {
    textEditingController.dispose();
    super.dispose();
  }

  void clearValue() {
    setValue(null);
    textEditingController.text = '';
  }

  @override
  void afterUpdateModel(FormeCupertinoTimerFieldModel old,
      FormeCupertinoTimerFieldModel current) {
    if (value == null) return;
    if (current.minuteInterval != null &&
        value!.inMinutes % current.minuteInterval! != 0) {
      clearValue();
    }
    if (value != null &&
        current.secondInterval != null &&
        value!.inSeconds % current.secondInterval! != 0) {
      clearValue();
    }
    if (value != null && current.formatter != null ||
        (current.mode != null && current.mode != old.mode)) {
      textEditingController.text = value == null
          ? ''
          : (current.formatter ?? _formatter)(value!, current.mode ?? mode);
    }
  }
}

class FormeCupertinoTimerFieldModel extends FormeModel {
  final FormeDurationFormatter? formatter;
  final FormeCupertinoTextFieldModel? textFieldModel;

  /// **it's a child of [CupertinoButton]**
  final Widget? confirmWidget;

  final CupertinoTimerPickerMode? mode;
  final int? minuteInterval;
  final int? secondInterval;
  final Alignment? alignment;
  final Color? backgroundColor;
  final FormeCupertinoModelPopupData? modelPopupData;

  FormeCupertinoTimerFieldModel({
    this.formatter,
    this.textFieldModel,
    this.confirmWidget,
    this.mode,
    this.minuteInterval,
    this.secondInterval,
    this.alignment,
    this.backgroundColor,
    this.modelPopupData,
  });
  @override
  FormeCupertinoTimerFieldModel copyWith(FormeModel oldModel) {
    FormeCupertinoTimerFieldModel old =
        oldModel as FormeCupertinoTimerFieldModel;
    return FormeCupertinoTimerFieldModel(
      formatter: formatter ?? old.formatter,
      textFieldModel:
          FormeCupertinoTextFieldModel.copy(old.textFieldModel, textFieldModel),
      confirmWidget: confirmWidget ?? old.confirmWidget,
      mode: mode ?? old.mode,
      minuteInterval: minuteInterval ?? old.minuteInterval,
      secondInterval: secondInterval ?? old.secondInterval,
      alignment: alignment ?? old.alignment,
      backgroundColor: backgroundColor ?? old.backgroundColor,
      modelPopupData:
          FormeCupertinoModelPopupData.copy(old.modelPopupData, modelPopupData),
    );
  }
}
